from PyQt4 import QtGui, QtCore
import os
import logging

# --------[ Backend ]---------
import backendinterface as bif
# --------[ Essentials ]---------
import storage
from settings import Settings
from backend import board
# --------[ Necessary Widgets ]------
import kcgwidget as kcgw
from controlwidget import ControlWidget
from multiWidget import MultiWidget
from groupedelements import MenuItems, Elements
from multipage import MultiPage
import bitsTable as bt
from base import log
# ---------[ Widgets IMPORTANT!!! ]------------------
# this enables widgets. If this is not imported (even though it is not directly used) no widgets will be available
from widgets import *
# from widgets import *  # copy in case the above line gets removed by ide
# ---------[ IMPORTANT ]---------------------

tr = kcgw.tr

import config
import time
import getpass

Elements.setFlags({'autoremove': True, 'warn': True, 'exception_on_deleted': False})

def readconfig(parent):
    """
    Reads the config and evalues certain variables
    Also: Validates config to check if all necessary values are there
    :param parent: parent for popup windows
    :return: -
    """

    nec_conf = ['acquireSettingsIcon', 'bunches_per_turn', 'default_log_entries', 'default_save_location', 'default_subdirectory_name',
                 'epics_base_path', 'epics_log_entry_pvs', 'force_ask', 'guiIcon', 'language', 'logCommentIcon', 'logIcon',
                 'newPlotDataIcon', 'newPlotLiveIcon', 'save_header', 'show_advanced_control', 'singleReadIcon', 'startIcon',
                 'stopIcon', 'style', 'tRev', 'timingIcon', 'epics_test_pv']
    missing_conf = []
    for c in nec_conf:
        if c not in dir(config):
            missing_conf.append(c)
    if missing_conf:
        class ConfigError(Exception):
            pass
        raise ConfigError('The Following variables are missing in config.py: "' + '", "'.join(missing_conf)+'"')

    if config.language != "en_GB":
        kcgw.translator.load('lang/'+ config.language)
    else:
        global tr
        kcgw.tr = lambda _, x: x
        tr = lambda _, x: x
    dateG = "{d}.{m}.{y}"
    dateGd = "{d}_{m}_{y}"
    dateA = "{m}-{d}-{y}"
    times = "{H}_{M}"
    timel = "{H}_{M}_{S}"
    session = ""
    if "{ask}" in config.default_subdirectory_name:
        status = False
        while not status:
            text, status = QtGui.QInputDialog.getText(parent, tr("Heading", "Subdirectory"),
                                                      tr("Dialog", "Enter a name for the Subdirectory\n"
                                                                            "in which data will be saved:\n"
                                                                            "NOTE: You are being asked because it "
                                                                            "was set this way in the config file."))
            if not status and not config.force_ask:
                config.default_subdirectory_name = "{user}_{dateGd}-{timel}"
                break
            else:
                config.subdirectory_name = text.replace(" ", "_")
                return
    if "{sessionname}" in config.default_subdirectory_name:
        status = False
        while not status:
            text, status = QtGui.QInputDialog.getText(parent, tr("Heading", "Sessionname"),
                                                      tr("Dialog", "Enter Sessionname\n"
                                                                           "NOTE: You are being asked because it "
                                                                           "was set this way in the config file.:"))
            if not status and not config.force_ask:
                config.default_subdirectory_name = "{user}_{dateGd}-{timel}"
                break
            else:
                session = text.replace(" ", "_")

    config.default_subdirectory_name = config.default_subdirectory_name.format(
        dateG=dateG, dateGd=dateGd, dateA=dateA, times=times, timel=timel,
        d=time.strftime("%d"), m=time.strftime("%m"), y=time.strftime("%y"),
        H=time.strftime("%H"), M=time.strftime("%M"), S=time.strftime("%S"),
        timestamp=time.localtime(), user=getpass.getuser(), sessionname=session
    )
    config.subdirectory_name = config.default_subdirectory_name.format(
        d=time.strftime("%d"), m=time.strftime("%m"), y=time.strftime("%y"),
        H=time.strftime("%H"), M=time.strftime("%M"), S=time.strftime("%S"),
        timestamp=time.localtime(), user=getpass.getuser()
    )
    if config.default_save_location == "pwd":
        import os
        config.save_location = os.getcwd()
    else:
        config.save_location = config.default_save_location


_MultiView_Name_ = "MultiView"

class CentralWidget(kcgw.KCGWidgets):
    """
    Central Widget for the KCG gui main window
    """
    def __init__(self, parent=None):
        super(CentralWidget, self).__init__(parent=parent)

        # -------[ Create empty Groups to avoid warnings ]---------
        MenuItems.createEmptyGroup('Setup/Control')
        MenuItems.createEmptyGroup('Bits Table')
        # -------[ END ]---------------

        self.layout = QtGui.QHBoxLayout()
        self.setLayout(self.layout)
        self.pagesWidget = MultiPage(self)
        self.layout.addWidget(self.pagesWidget)
        self.mainControlWidget = ControlWidget()
        self.pagesWidget.addPage(self.mainControlWidget, "Setup/Control")
        self.mainMultiWidget = MultiWidget()
        self.pagesWidget.addPage(self.mainMultiWidget, "MultiView")
        self.tableWidget = bt.AdvancedBoardInterface(parent=self)
        self.tableWidget.hide()



class Gui(QtGui.QMainWindow):
    """
    Main Window of the KCG gui
    """
    def __init__(self):
        super(Gui, self).__init__()

        self.createEmptyGroups()
        bif.bk_check_for_board()
        readconfig(self)

        # ----------[ Set Variables and create objects ]-----------------
        self.storage = storage.Storage()
        storage.storage = self.storage
        self.settings = None # Only create Window when used
        self.statusbar = self.statusBar()
        kcgw.statusbar = self.statusbar # set status bar to kcgw to easily access from other sources
        self.pageIndicator = QtGui.QLabel()
        self.statusbar.addPermanentWidget(self.pageIndicator)
        self.cw = CentralWidget(self)
        self.setCentralWidget(self.cw)

        self.doMenu()
        self.initUI()
        self.finalizeInit()

        self.after_start_status_handler()


    def initUI(self):
        """
        Initialize ui
        :return: -
        """
        self.setWindowTitle("KCG - Kapture Control Gui")
        self.setWindowIcon(QtGui.QIcon(config.guiIcon))

        self.statusbar.showMessage(board.status.status_text)

        # QtGui.QApplication.setStyle("Oxygen") # Make it look less blown up in Gnome for example

    def createEmptyGroups(self):
        # ------[ Create Groups to avoid warnings ]---------
        Elements.createEmptyGroup("acquire")
        Elements.createEmptyGroup("timing")
        Elements.createEmptyGroup("no_board")


    def finalizeInit(self):
        """
        Final things done at initialisation
        :return: -
        """


        self.populate_storage()

        with open("style/style.css") as f:
            styleSheet = f.read()
        if config.style == 'blue':
            with open('style/blue.css') as f:
                styleSheet += f.read()
        self.setStyleSheet(styleSheet)


        # evaluate config file regarding advanced_control
        self.showAdvancedControl(config.show_advanced_control)
        self.storage.advanced_control = config.show_advanced_control


        board.config.update('header', config.save_header)

        if not os.path.isdir(storage.storage.save_location + '/' + storage.storage.subdirname):
            os.makedirs(storage.storage.save_location + '/' + storage.storage.subdirname)
        self.measurementLogger = log.MeasurementLogger()  # storage.storage.save_location + '/' + storage.storage.subdirname
                                                       # + "/Measurement.log")
        log.logger = self.measurementLogger
        logStrings = []
        functionAndParameter = []
        for par in self.measurementLogger.predefined_parameters:  # get strings and functions in seperate lists
            logStrings.append(par[0])
            functionAndParameter.append(par[1])
        for e in config.default_log_entries:  # for every entry:
            if e in logStrings:
                self.measurementLogger.register_parameter(e, functionAndParameter[logStrings.index(e)][0], functionAndParameter[logStrings.index(e)][1])
        self.measurementLogger.register_dumper(board.config.dump)
        if log.noepics and log.epics_reachable:
            logging.info("Epics installation not found. Logfiles will not contain information that is to be"
                         "obtained via epics.")
        if not log.epics_reachable:
            logging.info("Epics PVs could not be accessed. Check Internet connection and Epics PV provider. Logfiles will not contain"
                        "information that is to be obtained via epics.")



    def doMenu(self):
        """
        Create and show the menu and it's entries
        :return: -
        """
        self.menu = self.menuBar()

        self.fileMenu = self.menu.addMenu("&"+tr("Button", "File"))
        self.settingsAction = self.fileMenu.addAction(tr("Button", "Settings"), self.showSettings, "Ctrl+P")
        self.quitAction = self.fileMenu.addAction(QtGui.QIcon("icons/exit.png"), tr("Button", "Quit"), self.close, "Ctrl+Q")
        self.menu.setCornerWidget(self.cw.pagesWidget.leftright)

        # ----------[ Page specific Menu Entries ]-------------
        self.multiMenu = self.menu.addMenu("&"+tr("Button", "Windows"))
        MenuItems.addMenuItem(_MultiView_Name_, self.multiMenu)
        self.plotAction = self.multiMenu.addAction(QtGui.QIcon(config.newPlotLiveIcon), tr("Button", "New Plot"), self.cw.mainMultiWidget.leftBar.add_plot)
        self.addWindowMenuEntries()

        self.acquireMenu = self.menu.addMenu("&"+tr("Button", "Acquire"))
        MenuItems.addMenuItem(_MultiView_Name_, self.acquireMenu)
        self.startAcquisitionAction = self.acquireMenu.addAction(QtGui.QIcon(config.startIcon), tr("Button", "Start Acquisition"), bif.bk_acquire)
        self.startAcquisitionAction.setObjectName("start_acquisition_action")

        MenuItems.addMenuItem("continuous_read", self.startAcquisitionAction)
        MenuItems.addMenuItem("acquireTrigger", self.startAcquisitionAction)

        # -----[ disable Menu Items for MultiView as it is not the startup page ]-------------
        # this could be avoided if menu is created before the multipage widget
        MenuItems.setEnabled(_MultiView_Name_, False)

        self.help = self.menu.addMenu("&"+tr("Button", "Help"))
        import webbrowser
        self.help.addAction(tr("Button", "Open Manual"), lambda: webbrowser.open("Documentation/build/html/index.html"))
        self.help.addAction(tr("Button", "About"), self.showAbout)


    def showAbout(self):
        version = open("VERSION").read()
        about = QtGui.QDialog(self) # TODO: read about text externally? read version externally?
        about.setWindowTitle("KCG - About")
        about_label = QtGui.QLabel(tr("About", "KCG - KAPTURE Control Gui\n\n"
                                               "This is the graphical control interface to the KAPTURE board\n\n"
                                               "KAPTURE - Karlsruhe Pulse-Taking and Ultrafast Readout Electronics\n\n"
                                               "Version: ")+version)
        about_layout = QtGui.QHBoxLayout()
        about.setLayout(about_layout)

        pxm = QtGui.QPixmap(config.guiIcon)
        icon_label = QtGui.QLabel("")
        icon_label.setPixmap(pxm.scaled(QtCore.QSize(64, 64), QtCore.Qt.KeepAspectRatio))

        about_layout.addWidget(icon_label)
        about_layout.addWidget(about_label)

        about.exec_()

    def addWindowMenuEntries(self):
        """
        Adds Window Menu entries for custom widgets
        :return: -
        """
        for f in kcgw.get_registered_widgets():
            self.multiMenu.addAction(*f[:3]) # TODO: icon - ???


    def showSettings(self):
        """
        Create and show settings window
        :return: -
        """
        if self.settings: # use preopened window
            self.settings.show()
            self.settings.raise_()
            self.settings.activateWindow()
        else:
            self.settings = Settings(self.storage)
            self.settings.changed.connect(self.updateSettings)

    def updateSettings(self, changedsettings):
        """
        Update settings in storage if settings were changed in the settings window.
        :param changedsettings: list of settings that have been changed
        :return: -
        """
        for setting in changedsettings:



            if setting == 'language':
                lang = getattr(self.storage, setting)
                self.update_configuration_file({'language':'"'+str(lang)+'"'})
                QtGui.QMessageBox.information(self, "Change Language", "Language change takes effect after Gui restart", 1)
            if setting == 'advanced_control':
                self.showAdvancedControl(getattr(self.storage, setting))
            if bif.bk_get_config(setting) != None:
                bif.bk_update_config(setting, getattr(self.storage, setting))

    def showAdvancedControl(self, value):
        """
        Enable or disable advanced table control view (Tables for registers)
        :param value: (bool) True to show and False to hide advanced view
        :return: -
        """
        if value:
            if self.cw.tableWidget.isHidden():
                self.cw.pagesWidget.addPage(self.cw.tableWidget, 'Bits Table', set_to_first=False)
                self.cw.tableWidget.show()
        else:
            if not self.cw.tableWidget.isHidden():
                self.cw.pagesWidget.removePage(self.cw.tableWidget)
                self.cw.tableWidget.hide()


    def after_start_status_handler(self):
            bif._bif_status_readout()

    def populate_storage(self):
        """
        Initially fills storage with predefined settings and configuration values
        :return: -
        """
        self.storage.header = config.save_header
        self.storage.subdirname = config.subdirectory_name
        self.storage.save_location = config.save_location
        self.storage.language = config.language
        self.storage.advanced_control = False

    def update_configuration_file(self, new_conf):
        """
        Update variablevalues in config file
        :param new_conf: Dictionary with variable, value pair
        :return:
        """
        import re
        filename = "config.py"
        RE = '(('+'|'.join(new_conf.keys())+')\s*=)[^\r\n]*?(\r?\n|\r)'
        pat = re.compile(RE)

        def jojo(mat,dic = new_conf ):
            return dic[mat.group(2)].join(mat.group(1,3))

        with open(filename,'rb') as f:
            content = f.read()

        with open(filename,'wb') as f:
            f.write(pat.sub(jojo,content))

    def closeEvent(self, ev):
        """
        Handles closing of the GUI - this function is called by pyqt upon a close event.
        Asks if user really wants to close the gui
        :param ev: event
        :return: -
        """
        cl = QtGui.QMessageBox.question(self, tr("Heading", "Close KCG"),
                                        tr("Dialog", "Close KCG?\nYou will loose the state of open plots etc."),
                                        QtGui.QMessageBox.No | QtGui.QMessageBox.Yes,
                                        QtGui.QMessageBox.No)
        if cl == QtGui.QMessageBox.Yes:
            if self.settings:
                self.settings.close()
            ev.accept()
        else:
            ev.ignore()

