"""
Class containing the creators for buttons etc
"""
from PyQt4 import QtGui, QtCore, QtSvg

tr = lambda _,x: x

class BigIconButton(QtGui.QPushButton):
    def __init__(self, path, width, height, connect=None, tooltip=None, parent=None):
        super(BigIconButton, self).__init__(parent)
        self.setIcon(QtGui.QIcon(path))
        self.setFixedSize(width, height)
        self.setIconSize(QtCore.QSize(width*0.7, height*0.7))
        if connect:
            self.clicked.connect(connect)
        if tooltip:
            self.setToolTip(tooltip)

class clickLabel(QtGui.QLabel):
    clicked = QtCore.pyqtSignal()
    def mouseReleaseEvent(self, QMouseEvent):
        self.clicked.emit()

class switchLabel(QtGui.QLabel):
    clicked = QtCore.pyqtSignal()
    def __init__(self, startRight=False): # self.state: True->Right False->Left
        super(switchLabel, self).__init__()
        self.leftSwitch = QtGui.QPixmap("icons/SwitchButtonLeft.png")
        self.leftSwitch = self.leftSwitch.scaled(40, 20, transformMode=QtCore.Qt.SmoothTransformation)
        self.rightSwitch = QtGui.QPixmap("icons/SwitchButtonRight.png")
        self.rightSwitch = self.rightSwitch.scaled(40, 20, transformMode=QtCore.Qt.SmoothTransformation)
        if startRight:
            self.setPixmap(self.rightSwitch)
            self.state = True
        else:
            self.setPixmap(self.leftSwitch)
            self.state = False
    def mouseReleaseEvent(self, QMouseEvent):
        if self.state:
            self.state = False
            self.setPixmap(self.leftSwitch)
        else:
            self.state = True
            self.setPixmap(self.rightSwitch)
        self.clicked.emit()


class ClickableSVG(QtGui.QWidget):
    clicked = QtCore.pyqtSignal()
    def __init__(self, path, width, height, wwidth=None, wheight=None, parent=None):
        super(ClickableSVG, self).__init__(parent)
        self.svg = QtSvg.QSvgWidget(path)
        self.svg.setFixedSize(width, height)
        layout = QtGui.QHBoxLayout()
        layout.addWidget(self.svg)
        self.setLayout(layout)
        if wwidth:
            self.setFixedWidth(wwidth)
        if wheight:
            self.setFixedHeight(wheight)
    def mouseReleaseEvent(self, QMouseEvent):
        self.clicked.emit()
    def changeSvg(self, path):
        self.svg.load(path)


class KCGWidgets(QtGui.QWidget):
    closeSignal = QtCore.pyqtSignal()
    def __init__(self, name=None, parent=None):
        super(KCGWidgets, self).__init__(parent)
        self._id = None
        self._type = None
        self._name = None
        if name:
            self.theName = name

    @property
    def theType(self):
        return self._type

    @property
    def theId(self):
        return self._id

    @property
    def theName(self):
        return self._name

    @theType.setter
    def theType(self, t):
        self._type = t # TODO: ??

    @theId.setter
    def theId(self, i):
        self._id = i # TODO: ??

    @theName.setter
    def theName(self, n):
        self._name = n # TODO: ??

    def createButton(self, text="", x=None, y=None, dimensions=None, tooltip="", connect=False, icon=None):
        button = QtGui.QPushButton(text, self)
        if tooltip:
            button.setToolTip(tooltip)
        if not dimensions:
            button.resize(button.sizeHint())
        else:
            button.resize(dimensions)
        if x and y:
            button.move(x, y)
        if connect:
            button.clicked.connect(connect)
        if icon:
            button.setIcon(icon)
        return button

    def createLabel(self, text=None, image=None, tooltip=None, click=False, connect=None):
        if click:
            label = clickLabel(self)
            if connect:
                label.clicked.connect(connect)
        else:
            label = QtGui.QLabel(self)
        if text:
            label.setText(text)
        if image:
            label.setPixmap(image)
        if tooltip:
            label.setToolTip(tooltip)
        return label

    def createSpinbox(self, min, max, interval=1, start_value=0, connect=None):
        """
        create a Spinbox
        :param min: (int) minimum Value
        :param max: (int) maximum Value
        :param interval: (int) interval
        :param start_value: (int) start Value
        :param connect: (callable) function to call on value change
        :return: -
        """
        spinbox = QtGui.QSpinBox()
        spinbox.setRange(min, max)
        spinbox.setSingleStep(interval)
        spinbox.setValue(start_value)
        if connect:
            spinbox.valueChanged.connect(connect)
        return spinbox

    def createInput(self, text=None, read_only=False, width=None):
        """
        Create Input
        :param text: (str) Default Text
        :param read_only: (bool) set input as read only
        :param width: (int) width of the input field in pixels
        :return: -
        """
        input = QtGui.QLineEdit()
        if text:
            input.setText(text)
        if width:
            input.setMinimumWidth(width)
        input.setReadOnly(read_only)
        return input

    def createCheckbox(self, text="", tooltip="", checked=False, connect=None):
        """
        Create Checkbox
        :param tooltip: (str) what tooltip text to display
        :param checked: (bool) Checkt as default?
        :param connect: (callable) function to connect
        :return:
        """
        cb = QtGui.QCheckBox(text)
        cb.setToolTip(tooltip)
        cb.setChecked(checked)
        if connect:
            cb.clicked.connect(connect)
        return cb

    def createSwitch(self, startRight=False, connect=None):
        sw = switchLabel(startRight)
        if connect:
            sw.clicked.connect(connect)
        return sw

    def closeEvent(self, event):
        super(KCGWidgets, self).closeEvent(event)
        self.closeSignal.emit()



class KCGSubWidget(QtGui.QMdiSubWindow):
    _id = None
    _name = None
    _type = None

    def __init__(self, name=None, unique_id=None, typ=None, minSize=False):
        super(KCGSubWidget, self).__init__()
        if name != None:
            self.theName = name
        if unique_id != None:
            self.theId = unique_id
        if typ != None:
            self.theType = typ



    @property
    def theType(self):
        return self._type

    @property
    def theId(self):
        return self._id

    @property
    def theName(self):
        return self._name

    @theType.setter
    def theType(self, t):
        self._type = t # TODO: ??

    @theId.setter
    def theId(self, i):
        self._id = i # TODO: ??

    @theName.setter
    def theName(self, n):
        self._name = n # TODO: ??


class MultilineInputDialog(QtGui.QDialog):
    def __init__(self):
        super(MultilineInputDialog, self).__init__()
        self.answer = False
    def fill(self, heading, label):
        self.setWindowTitle(heading)
        self.layout = QtGui.QVBoxLayout()
        self.setLayout(self.layout)

        self.tl = QtGui.QLabel(label)
        self.layout.addWidget(self.tl)
        self.ti = QtGui.QPlainTextEdit()
        self.layout.addWidget(self.ti)

        # Advanced method to activate function call on Ctrl+Return (in this case the ok function aka press ok button)
        # def ctrlEnter(event):
        #     if event.key() == QtCore.Qt.Key_Return:
        #         if event.modifiers() == QtCore.Qt.ControlModifier:
        #             self.ok()
        #     QtGui.QPlainTextEdit.keyPressEvent(self.ti, event)
        # self.ti.keyPressEvent = ctrlEnter

        self.blayout = QtGui.QHBoxLayout()
        self.layout.addLayout(self.blayout)
        self.okButton = QtGui.QPushButton(tr("Button", "OK"))
        self.okButton.pressed.connect(self.ok)
        self.okButton.setShortcut("Ctrl+Return")
        self.cancelButton = QtGui.QPushButton(tr("Button", "Cancel"))
        self.cancelButton.pressed.connect(self.cancel)
        self.cancelButton.setShortcut("Esc")
        self.blayout.addWidget(self.okButton)
        self.blayout.addWidget(self.cancelButton)

    def ok(self):
        self.answer = True
        self.close()

    def cancel(self):
        self.answer = False
        self.close()

    def get_text(self, heading, label):
        self.fill(heading, label)
        self.exec_()
        return unicode(self.ti.toPlainText()), self.answer



class IdGenerator():
    """
    Generate Unique Id for every subwindow
    """
    highest_id = 0
    def genid(self):
        self.highest_id += 1
        return self.highest_id

idg = IdGenerator()  # declare idgenerator instance to use globally


_registered_possible_widgets = []
_registered_possible_widget_functions = []
def register_widget_creation_function(creation_func):
    _registered_possible_widget_functions.append(creation_func)

def register_widget(icon, text, target, shortcut=None):
    _registered_possible_widgets.append([icon, text, target, shortcut])

def get_registered_widgets():
    return _registered_possible_widgets

def get_registered_widget_functions():
    return _registered_possible_widget_functions

consistency_updater = None  # To be set in LeftBar

