# -*- coding: utf-8 -*-
from PyQt4 import QtGui, QtCore

import kcgwidget as kcgw
from plotWidget import PlotWidget
from backend import board
from backend import io
import storage
import widgets.acquiresettings as acqs
from groupedelements import Buttons, Elements
import backendinterface as bif
import log


tr = kcgw.tr

import config

LIVE = 1
FILE = 2

class AcquisitionAndInfo(kcgw.KCGWidgets):
    """
    Widget to show below the plot list.
    Show certain information.
    Provide Start Acquisition button and settings.
    """
    def __init__(self):
        super(AcquisitionAndInfo, self).__init__()
        self.consistency = QtGui.QHBoxLayout()
        self.consistency_label = self.createLabel("Cons.:")
        self.consistency_content = self.createLabel(u"●")
        self.consistency_content.setStyleSheet("color: lightgrey;")
        self.consistency.addWidget(self.consistency_label)
        self.consistency.addWidget(self.consistency_content)

        self.temperature = self.createLabel(tr("Label", "Temp"))
        self.skipturns = self.createLabel(
            tr("Label", "O_skip:") + " " + str(bif.bk_get_config('orbits_skip')),
            tooltip=tr("Tooltip", "Skipped orbits\nKlick to open acquire settings"),
            click=True,
            connect=acqs.addAcquireSettingsWidgets
        )
        board.config.observe(self.skipturns, lambda x: self.skipturns.setText(tr("Label", "O_skip:")+" "+str(x)), 'orbits_skip')
        self.orbitsobserved = self.createLabel(
            tr("Label", "O_obs:") + " " + str(bif.bk_get_config('orbits_observe')),
            tooltip=tr("Tooltip", "Number of observed Orbits\nKlick to open acquire settings"),
            click=True,
            connect=acqs.addAcquireSettingsWidgets
        )
        board.config.observe(self.orbitsobserved, lambda x: self.orbitsobserved.setText(tr("Label", "O_obs:") + " " + str(x)), 'orbits_observe')
        self.orbitscount = self.createLabel(
            tr("Label", "Count:") + " " + str(bif.bk_get_config('orbits_count')),
            tooltip=tr("Tooltip", "Number of acquisitions\nKlick to open acquire settings"),
            click=True,
            connect=acqs.addAcquireSettingsWidgets
        )
        board.config.observe(self.orbitscount, lambda x: self.orbitscount.setText(tr("Label", "Count:")+ " " + str(x)), 'orbits_count')
        self.orbitswait= self.createLabel(
            tr("Label", "T_wait:") + " " + str(bif.bk_get_config('orbits_wait_time')),
            tooltip=tr("Tooltip", "Time in seconds to wait between acquisitions\nKlick to open acquire settings"),
            click=True,
            connect=acqs.addAcquireSettingsWidgets
        )
        board.config.observe(self.orbitswait, lambda x: self.orbitswait.setText(tr("Label", "T_wait:")+ " " + str(x)), 'orbits_wait_time')
        self.acquire = self.createButton(tr("Button", "Start Acquisition"), icon=QtGui.QIcon(config.startIcon), connect=bif.bk_acquire)

        self.log = self.createButton(tr("Button", "Log"), icon=QtGui.QIcon(config.logIcon), connect=lambda: log.log(additional="Manual Log"), tooltip="Rightclick to Configure")
        self.log_w_comment = self.createButton(tr("Button", "Log"), icon=QtGui.QIcon(config.logCommentIcon), connect=self.do_log_w_comment, tooltip="Log with Comment\nRightclick to Configure")
        self.log.contextMenuEvent = self.log_configure_context_menu
        self.log_w_comment.contextMenuEvent = self.log_configure_context_menu

        self.acquireShortcut = QtGui.QShortcut("Ctrl+B", self, bif.bk_acquire)
        Buttons.addButton("continuous_read", self.acquireShortcut)
        Buttons.addButton("continuous_read", self.acquire)
        Buttons.addButton("acquireTrigger", self.acquire)
        Elements.addItem('no_board', Elements.getElements('acquireTrigger'))
        self.overLayout = QtGui.QVBoxLayout()
        self.setLayout(self.overLayout)
        self.layout = QtGui.QGridLayout() # TODO: add text for build_spectrograms ?
        self.layout.addLayout(self.consistency, 0, 0)
        self.layout.addWidget(self.temperature, 0, 1)
        self.layout.addWidget(self.skipturns, 1, 1)
        self.layout.addWidget(self.orbitsobserved, 1, 0)
        self.layout.addWidget(self.orbitscount, 2, 0)
        self.layout.addWidget(self.orbitswait, 2, 1)
        # self.layout.addWidget(self.acquire, 2, 1)
        self.overLayout.addLayout(self.layout)
        self.overLayout.addWidget(self.acquire)

        self.logLayout = QtGui.QHBoxLayout()
        self.overLayout.addLayout(self.logLayout)
        self.logLayout.addWidget(self.log)
        self.logLayout.addWidget(self.log_w_comment)


        def update_temp():
            if board.status.board_connected:
                self.temperature.setText("Temp: " + bif.bk_get_temperature() + u" °C")
        try:
            update_temp()
        except:
            print "cannot get temperature"

        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(update_temp)
        self.timer.start(30000)
        kcgw.consistency_updater = self.update_consistency

    def do_log_w_comment(self):
        # text, ok = QtGui.QInputDialog.getText(self, tr("Heading", 'Log Comment'), tr("Label", 'Log Comment:'))
        text, ok = kcgw.MultilineInputDialog().get_text(tr("Heading", "Log Comment"), tr("Label", "Log Comment:"))
        if ok:
            log.log(additional=text)

    def log_configure_context_menu(self, event):
        pos = event.globalPos()
        def configure_log():
            c = log.ConfigureLog(self)
            c.exec_()
            del c
        if pos is not None:
            menu = QtGui.QMenu(self)
            configure = menu.addAction(tr("Button", "Configure Log"))
            configure.triggered.connect(configure_log)
            menu.popup(pos)
        event.accept()


    def update_consistency(self, status):
        """
        Set consistency indicator
        :param status: True if consistent, False if inconsistent, None is nothing
        :return: -
        """
        if status == True:
            self.consistency_content.setStyleSheet("color: green;")
        elif status == False:
            self.consistency_content.setStyleSheet("color: red;")
        else:
            self.consistency_content.setStyleSheet("color: lightgrey;")


class LeftBar(kcgw.KCGWidgets):
    """
    Left bar in the main view of the gui.
    Shows plot list and acquisition and info panel
    """
    # general notes:
    # uid = unique_id
    # usid = unique_sub_id
    # upid = unique_parent_id
    possiblePlots = ["Heatmap", "FFT", "Trains", "Combined"]

    def __init__(self, parent):
        super(LeftBar, self).__init__("LeftBar")
        self.parent = parent

        self.openData = {}
        self.openDataNames = {}
        self.openPlots = {}
        self.openPlotWindows = {}
        self.Data = {}

        self.initUI()

    def initUI(self):
        self.layout = QtGui.QVBoxLayout()
        self.setMinimumWidth(200)

        self.treeWidget = QtGui.QTreeWidget()
        #        self.treeWidget.setStyleSheet(" QTreeView::branch { background: palette(base); }")
        self.treeWidget.setColumnCount(2)
        self.treeWidget.resizeColumnToContents(1)
#        self.treeWidget.header().resizeSection(0, 110)
        self.treeWidget.setHeaderHidden(True)
        self.treeWidget.itemClicked.connect(self.plot_clicked)
        self.treeWidget.contextMenuEvent = self.contextMenuEventListView
        self.rootItem = self.treeWidget.invisibleRootItem()

        self.info = AcquisitionAndInfo() # TODO: info bad/unclear variable name

        # self.layout.addWidget(self.createLabel("Plots"))
        self.layout.addWidget(self.treeWidget)
        self.layout.addWidget(self.info)

        self.setLayout(self.layout)


    def contextMenuEventListView(self, event):
        """
        Gets called when right mouse button is clicked
        :param event: the event that causes the call of this function
        :return: -
        """
        pos = event.globalPos()
        try:
            item = self.treeWidget.selectedItems()[0]
        except IndexError:
            return
        if pos is not None:
            menu = QtGui.QMenu(self)
            close = menu.addAction(tr("Button", "Close"))
            if item.is_child:
                close.triggered.connect(lambda: self.remove_plot(item.usid, silent=False, closePlotWindow=True))
            else:
                close.triggered.connect(lambda: self.remove_plot_tree(item.uid))
            menu.popup(pos)
        event.accept()

    def plot_clicked(self, item, i):
        """
        Function to handle when a plot item is clicked.
        :param item: what item is clicked
        :param i: what column
        :return: -
        """
        if item.is_child:
            self.openPlotWindows[item.usid].setFocus()
        else:
            if not item.ready:
                return
            if i==1:
                position = QtGui.QCursor.pos()
                menu = QtGui.QMenu(self)

                heatmap = menu.addAction(tr("Button", "Heatmap"))
                fft = menu.addAction(tr("Button", "FFT"))
                trains = menu.addAction(tr("Button", "Trains"))
                combined = menu.addAction(tr("Button", "Combined"))

                heatmap.triggered.connect(lambda: self.add_plot_node(text=tr("Label", 'Heatmap'), unique_id=item.uid, type=0, datatype=item.datatype))
                fft.triggered.connect(lambda: self.add_plot_node(text=tr("Label", 'FFT'), unique_id=item.uid, type=1, datatype=item.datatype))
                trains.triggered.connect(lambda: self.add_plot_node(text=tr("Label", 'Trains'), unique_id=item.uid, type=2, datatype=item.datatype))
                combined.triggered.connect(lambda: self.add_plot_node(text=tr("Label", 'Combined'), unique_id=item.uid, type=3, datatype=item.datatype))

                menu.popup(position)

    def add_plot_tree(self, text=False, unique_id=None, datatype=None):
        """
        Add a top node to the plot list.
        :param text: (str) text to be displayed in the plot list
        :param unique_id: (int) unique id of this top node
        :param datatype: (int) live or data from file
        :return: -
        """
        if datatype == FILE:
            fName = QtGui.QFileDialog.getOpenFileName()
            if fName == "":
                return
            if fName in self.openDataNames.itervalues():
                return
            self.openDataNames[unique_id] = fName
        else:
            if text in self.openDataNames.itervalues():
                return
            self.openDataNames[unique_id] = QtCore.QString(text) # Add a qstring for comparison
        self.openData[unique_id] = QtGui.QTreeWidgetItem()
        self.openData[unique_id].uid = unique_id
        self.openData[unique_id].is_child = False
        self.openData[unique_id].datatype = datatype
        if datatype == FILE:
            self.openData[unique_id].fName = fName
        self.openData[unique_id].setText(0, "Live" if datatype == LIVE else fName.split('/')[-1])
        self.openData[unique_id].setText(1, "Reading")
        if datatype == LIVE: # Insert LIVE on top
            self.treeWidget.insertTopLevelItem(0, self.openData[unique_id])
        else:
            self.treeWidget.addTopLevelItem(self.openData[unique_id])
        self.treeWidget.expandItem(self.openData[unique_id])
        self.treeWidget.resizeColumnToContents(0)
        self.openData[unique_id].ready = False  # indicates whether data was completely read or not
        QtGui.qApp.processEvents()
        try:
            self.read_data(datatype, unique_id, fName if datatype == FILE else None)
        except:
            self.Data[unique_id] = None
            self.remove_plot_tree(unique_id, silent=True)
            QtGui.QMessageBox.critical(self, tr("Heading", "Error Reading Data"), tr("Dialog", "There was an error reading the datafile, make shure it is valid and try again."))
        self.openData[unique_id].ready = True # indicates whether data was completely read or not
        self.openData[unique_id].setText(1, "+")

    def read_data(self, type, uid, fName):
        """
        Reads datafile
        :param type: FILE or LIVE - type of datasource
        :param uid: unique id for the treeelement
        :param fName: filename if type is FILE
        :return: -
        """
        if type == FILE:
            self.Data[uid] = io.read_from_file(fName, header=storage.storage.header)
        else:
            self.Data[uid] = bif.livePlotData


    def add_plot_node(self, text=False, unique_id=None, type=None, datatype=None):
        """
        Actually open a new plot window.
        :param text: (str) text that is displayed in the plot list
        :param unique_id: (int) unique id of this plot window
        :param type: (int) type of this plot window
        :param datatype: (int) live or data from file
        :return: -
        """
        unique_sub_id = kcgw.idg.genid()
        nr = self.openData[unique_id].childCount()
        if datatype == FILE:
            fName = self.openData[unique_id].fName
        else:
            fName = None
        name = "Live" if not fName else fName.split('/')[-1]
        self.openPlotWindows[unique_sub_id] = PlotWidget(name=name, unique_id=unique_sub_id, type=type, datatype=datatype, prefix=nr, fName=fName, data=self.Data[unique_id])
        self.openPlotWindows[unique_sub_id].close_signal.connect(lambda: self.remove_plot(unique_sub_id))
        self.openPlotWindows[unique_sub_id].change_type_signal.connect(self.update_plot)
        self.openPlotWindows[unique_sub_id].upid = unique_id
        self.parent.area.newWidget(self.openPlotWindows[unique_sub_id], name=text, unique_id=unique_sub_id, widget_type=type)
        self.openPlots[unique_sub_id] = QtGui.QTreeWidgetItem()
        self.openPlots[unique_sub_id].setText(0, str(nr)+"."+self.possiblePlots[type] + " - ADC 1")
        self.openPlots[unique_sub_id].is_child = True
        self.openPlots[unique_sub_id].usid = unique_sub_id
        self.openPlots[unique_sub_id].nr = nr
        self.openData[unique_id].addChild(self.openPlots[unique_sub_id])
        self.treeWidget.resizeColumnToContents(0)

    @QtCore.pyqtSlot()
    def remove_plot(self, unique_id, silent=False, closePlotWindow=False):
        """
        Removes a plot from the plot list.
        :param unique_id: (int) the unique id of the plot to remove
        :param silent: (bool) Ask to close the plot source node in the list if the last plot window is closed (if False the source node will not be closed)
        :param closePlotWindow: (bool) close the corresponding plot window
        :return: -
        """
        parent = self.openData[self.openPlotWindows[unique_id].upid]
        parent.removeChild(self.openPlots[unique_id])
        if parent.childCount() == 0 and not silent:
            reply = QtGui.QMessageBox.question(self, tr("Heading", 'Close Plot'), tr("Dialog", "Close this plot source?"),
                                               QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
                                               QtGui.QMessageBox.No)

            if reply == QtGui.QMessageBox.Yes:
                # self.rootItem.removeChild(parent)
                self.remove_plot_tree(parent.uid, silent=True)
        if closePlotWindow:
            self.openPlotWindows[unique_id].close_signal.disconnect()
            self.openPlotWindows[unique_id].close()
        self.openPlotWindows[unique_id] = None
        self.openPlots[unique_id] = None
        del self.openPlotWindows[unique_id]
        del self.openPlots[unique_id]


    def remove_plot_tree(self, unique_id, silent=False):
        """
        Remove the top node from the plot list
        :param unique_id: (int) the id of the top node
        :param silent: (bool) ask to close even if corresponding plot windows are open
        :return: -
        """
        if self.openData[unique_id].childCount() != 0 and not silent:
            reply = QtGui.QMessageBox.question(self, tr("Heading", 'Close Data Source'), tr("Dialog", "There are open plot windows.\n"
                                               "Close this plot source and all corresponding plot windows?"),
                                               QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
                                               QtGui.QMessageBox.No)
            if reply == QtGui.QMessageBox.No:
                return
        for plot in self.openData[unique_id].takeChildren():
            self.remove_plot(plot.usid, silent=True, closePlotWindow=True)
            self.openData[unique_id].removeChild(plot)
        self.rootItem.removeChild(self.openData[unique_id])
        del self.openData[unique_id]
        del self.openDataNames[unique_id]
        del self.Data[unique_id]

    @QtCore.pyqtSlot(int, int, int)
    def update_plot(self, uid, type, adc):
        """
        Updates the plot list when the type of a plot window changes
        :param uid: (int) unique id of the plot window to update
        :param type: (int) type of that plot window
        :return: -
        """
        self.openPlots[uid].setText(0, str(self.openPlots[uid].nr)+"."+self.possiblePlots[type] + " - ADC " + str(adc))
        self.treeWidget.resizeColumnToContents(0)

    def handle_dialog(self, value, diag=None,):
        """
        Function that handles the return of the dialog that asks for live or data from file
        :param value: (int) the value of the pressed button in the dialog
        :param diag: (QDialog) the dialog that is to be closed (optional)
        :return: -
        """
        if diag:
            diag.close()
        s = kcgw.idg.genid()
        if value == 1:
            self.add_plot_tree(text=tr("Label", "Live"), unique_id=s, datatype=LIVE)
        elif value == 2:
            self.add_plot_tree(text=tr("Label", "File"), unique_id=s, datatype=FILE)
        else:
            pass

    def add_plot(self, type=None):
        """
        Handles the creation of a plot.
        Also shows the dialog that asks for the data source.
        :return: -
        """
        if type == LIVE or type == FILE:
            self.handle_dialog(value=type)
            return
        self.ask = QtGui.QDialog()
        window = self.parent.geometry()
        askLayout = QtGui.QVBoxLayout()
        askLayout.addWidget(QtGui.QLabel(tr("Dialog", "Open file from disk or read live data?")))

        buttonLayout = QtGui.QHBoxLayout()
        askButtonLive = QtGui.QPushButton(tr("Button", "Live"))
        askButtonFile = QtGui.QPushButton(tr("Button", "Open File"))
        askButtonCancel = QtGui.QPushButton(tr("Button", "Cancel"))

        askButtonLive.pressed.connect(lambda: self.handle_dialog(diag=self.ask, value=1))
        askButtonFile.pressed.connect(lambda: self.handle_dialog(diag=self.ask, value=2))
        askButtonCancel.pressed.connect(lambda: self.handle_dialog(diag=self.ask, value=3))

        buttonLayout.addWidget(askButtonLive)
        buttonLayout.addWidget(askButtonFile)
        buttonLayout.addWidget(askButtonCancel)

        askLayout.addLayout(buttonLayout)
        self.ask.setGeometry(window.width()/2.-100, window.height()/2.-50, 200, 100)

        self.ask.setLayout(askLayout)
        self.ask.exec_()


