from PyQt4 import QtGui, QtCore

import kcgwidget as kcgw
from leftbar import LeftBar
import leftbar
import config

tr = kcgw.tr

class WidgetTypeError(Exception):
    pass

class MDIArea(kcgw.KCGWidgets):
    def __init__(self, parent):
        super(MDIArea, self).__init__("MDIArea")
        self.parent = parent

        self.widgets = {}
        self.initArea()

    def initArea(self):
        self.area = QtGui.QMdiArea()

    def newWidget(self, widget, name, unique_id, widget_type, minSize=False):
        if not isinstance(widget, kcgw.KCGWidgets):
            raise WidgetTypeError("Newly Created Widget is of type " +
                                  type(widget).__name__ +
                                  ". Expected class derived of " +
                                  kcgw.KCGWidgets.__name__ + ".")
        subWidget = kcgw.KCGSubWidget(name=name, unique_id=unique_id, typ=widget_type, minSize=minSize)
        subWidget.setAttribute(QtCore.Qt.WA_DeleteOnClose)
        widget.setParent(subWidget)

        if minSize:
            subWidget.resize(widget.minimumSizeHint()*1.2)


        widget.closeSignal.connect(subWidget.close)
        subWidget.setWidget(widget)
        self.area.addSubWindow(subWidget)
        subWidget.show()
        self.widgets[unique_id] = widget

class MultiWidget(QtGui.QWidget):
    def __init__(self):
        super(MultiWidget, self).__init__()
        self.layout = QtGui.QVBoxLayout()
        self.setLayout(self.layout)

        self.splitter = QtGui.QSplitter(QtCore.Qt.Horizontal)

        self.leftBar = LeftBar(self)
        self.area = MDIArea(self)
        self.toolbar = QtGui.QToolBar()
        kcgw.area = self.area

        # --------[ ToolBar ]--------
        self.new_live_action = QtGui.QAction(QtGui.QIcon(config.newPlotLiveIcon), tr("Button", "New Live Plot"), self)
        self.new_live_action.setShortcut("Ctrl+L")
        self.toolbar.addAction(self.new_live_action)
        self.new_live_action.triggered.connect(lambda: self.leftBar.add_plot(leftbar.LIVE))
        self.new_live_action.setToolTip(tr("Button", "New Live Plot") + "\n(Ctrl+L)")

        self.new_data_action = QtGui.QAction(QtGui.QIcon(config.newPlotDataIcon), tr("Button", "New Data Plot"), self)
        self.new_data_action.setShortcut("Ctrl+D")
        self.toolbar.addAction(self.new_data_action)
        self.new_data_action.triggered.connect(lambda: self.leftBar.add_plot(leftbar.FILE))
        self.new_data_action.setToolTip(tr("Button", "New Data Plot") + "\n(Ctrl+L)")

        self.layout.addWidget(self.toolbar)
        # ------[ End Tool Bar ]-----

        self.splitter.addWidget(self.leftBar)
        self.splitter.addWidget(self.area.area)
        self.layout.addWidget(self.splitter)

        self.evaluate_registered_widgets()
        self.evaluate_registered_widget_functions()

    def addToolbarButton(self, icon, text, target, shortcut=None):
        na = QtGui.QAction(icon, text, self)
        if shortcut:
            na.setShortcut(shortcut)
            na.setToolTip(text + "\n("+shortcut+")")
        self.toolbar.addAction(na)
        na.triggered.connect(target)

    def evaluate_registered_widgets(self):
        for f in kcgw.get_registered_widgets():
            self.addToolbarButton(*f)

    def evaluate_registered_widget_functions(self):
        for f in kcgw.get_registered_widget_functions():
            f()

