import pyqtgraph as pg
pg.setConfigOption('useOpenGl', True)
pg.setConfigOption('background', 'w')
pg.setConfigOption('foreground', 'k')
from threading import Thread
import numpy as np
from PyQt4 import QtCore, QtGui
import logging
from groupedelements import live_plot_windows
from backend import io
from backend import board

import kcgwidget as kcgw
tr = kcgw.tr
# from backend.dataset import DataSet
# from backend import io
# import storage

LIVE = 1
FILE = 2

class Enum():
    def __init__(self, *args):
        self.idx = 0
        for i in args:
            setattr(self, i, self.idx)
            self.idx += 1

class ThreadFFT(Thread):
    def __init__(self, data, callback_signal):
        self.data = data
        self.sig = callback_signal
        super(ThreadFFT, self).__init__()
    def run(self):
        freq = np.fft.rfftfreq(self.data.shape[0])
        self.sig.emit(np.fft.rfft(self.data), (min(freq), max(freq)))



plotList = [tr("Label", "Heatmap"), tr("Label", "FFT"), tr("Label", "Trains"), tr("Label", "Combined")]
PlotType = Enum(*[str(i) for i in plotList])

class SpectrogramColorLegendItem(pg.GraphicsWidget):
    gradientChanged = QtCore.pyqtSignal()
    def __init__(self, img=None):
        super(SpectrogramColorLegendItem, self).__init__()
        self.layout = QtGui.QGraphicsGridLayout()
        self.setLayout(self.layout)
        self.img = img
        self.gei = pg.GradientEditorItem(orientation='right')
        self.legend_axis = pg.AxisItem(orientation='left')
        # self.addItem(self.legend_axis)
        self.layout.addItem(self.legend_axis, 0, 0, alignment=QtCore.Qt.AlignVCenter)
        # self.addItem(self.gei)
        self.layout.addItem(self.gei, 0, 1)
        self.gei.sigGradientChanged.connect(self.gradient_changed)
        self.gei.loadPreset('spectrum')
        # self.setFixedWidth(80)
        self.image_changed()

    def gradient_changed(self):
        self.gradientChanged.emit()

    def set_image(self, img):
        self.img = img

    def update_axis(self):
        if self.img.getLevels():
            min, max = self.img.getLevels()
            self.legend_axis.setRange(min, max)

    def reset_gradient(self):
        self.gei.loadPreset('spectrum')

    def image_changed(self, reset_gradient=True):
        if self.img:
            self.update_axis()
            if reset_gradient:
                self.reset_gradient()

    def getLookupTable(self, nPts, alpha=None):
        return self.gei.getLookupTable(nPts, alpha=alpha)

    def resizeEvent(self, event):
        self.legend_axis.setHeight(self.gei.length)

class SubPlotWidget(pg.GraphicsLayoutWidget):
    def __init__(self):
        super(SubPlotWidget, self).__init__()

        # self.setStyleSheet("border: 5px solid black; margin: 10px;")
        self.plotType = 1
        self.plotItem = pg.PlotItem()
        self.img = pg.ImageItem()

        self.gradient_legend = SpectrogramColorLegendItem(self.img)
        def changelut():
            self.img.setLookupTable(self.gradient_legend.getLookupTable(512))
        self.gradient_legend.gradientChanged.connect(changelut)

        self.addItem(self.plotItem)
        self.plotItem.addItem(self.img)
        self.plotItemPlotScatter = pg.PlotDataItem(pen=None, symbolPen=pg.mkPen(50, 50, 255), symbolSize=3, pxMode=True, symbolBrush=pg.mkBrush(50, 50, 255), downsample=200)
        self.plotItemPlotScatter1 = pg.PlotDataItem(pen=None, symbolPen=pg.mkPen(255, 50, 50), symbolSize=3, pxMode=True, symbolBrush=pg.mkBrush(255, 50, 50), downsample=200)
        # self.plotItemPlotScatter2 = pg.PlotDataItem(pen=None, symbolPen=pg.mkPen(50, 255, 50), symbolSize=3, pxMode=True, symbolBrush=pg.mkBrush(50, 255, 50), downsample=200)
        self.plotItemPlot = pg.PlotDataItem()  # pen="#000000")
        self.plotItem.addItem(self.plotItemPlot)
        self.plotItem.addItem(self.plotItemPlotScatter)
        self.plotItem.addItem(self.plotItemPlotScatter1)
        # self.plotItem.addItem(self.plotItemPlotScatter2)

        # pos = np.array([0, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 1])
        # color = np.array([[0, 0, 255, 255], [0, 255, 255, 255], [0, 255, 0, 255], [130, 255, 0, 255], [255, 255, 0, 255], [255, 180, 0, 255], [255, 100, 0, 255], [255, 0, 0, 255]])
        # cmap = pg.ColorMap(pos, color)
        # self.lut = cmap.getLookupTable(0.0, 1.0, 256)
        # self.img.setLookupTable(self.lut)


    @QtCore.pyqtSlot(np.ndarray, tuple, tuple)
    def plot(self, data, xvalueborders=None, yvalueborders=None, autorange=True):
        self.plotItemPlot.clear()
        self.plotItemPlotScatter.clear()
        self.plotItemPlotScatter1.clear()
        # self.plotItemPlotScatter2.clear()
        self.plotItem.resetTransform()
        self.plotItem.getAxis('bottom').setScale()

        if self.plotType == PlotType.FFT or self.plotType == PlotType.Heatmap:
            self.addItem(self.gradient_legend)
            self.gradient_legend.show()

            # self.parent().max_slider.setMinimum(np.min(data))
            # self.parent().max_slider.setMaximum(np.max(data))
            # self.parent().low_slider.setMinimum(np.min(data))
            # self.parent().low_slider.setMaximum(np.max(data))


            self.img.setImage(data, scale=[1, 1/1000.])
            self.img.resetTransform()
            # self.parent().max_slider.setValue(self.img.getLevels()[1])
            # self.parent().low_slider.setValue(self.img.getLevels()[0])
            self.img.setAutoDownsample(True)
            if autorange:
                self.gradient_legend.image_changed()
                pass

            if xvalueborders:
                self.plotItem.getAxis('bottom').setScale((xvalueborders[1]-xvalueborders[0])/float(data.shape[0]))
                self.img.translate(xvalueborders[0]/self.plotItem.getAxis('bottom').scale, 0)

            else:
                self.plotItem.getAxis('bottom').setScale()

            if yvalueborders:
                self.img.translate(0, yvalueborders[0])
                self.plotItem.getAxis('left').setScale(yvalueborders[1]/float(data.shape[1]))

            self.plotItem.setClipToView(True)

        if self.plotType == PlotType.Trains:
            self.img.clear()
            self.gradient_legend.hide()
            try:
                self.removeItem(self.gradient_legend)
                pass
            except Exception, e:
                if not "Could not determine index of item" in str(e):
                    raise
            self.plotItemPlotScatter.clear()
            self.plotItemPlotScatter1.clear()
            # self.plotItemPlotScatter2.clear()
            self.plotItemPlot.setData(data)
            bounds = [np.min(data), np.max(data)]
            self.plotItemPlot.getViewBox().setRange(xRange=[0, len(data)], # Do this twice because the first wont update the image
                yRange=[bounds[0]-0.1*(bounds[1]-bounds[1]), bounds[1]+0.1*(bounds[1]-bounds[1])], update=True)
            self.plotItemPlot.getViewBox().setRange(xRange=[0, len(data)],
                yRange=[bounds[0]-0.1*(bounds[1]-bounds[1]), bounds[1]+0.1*(bounds[1]-bounds[1])], update=True)
        if self.plotType == PlotType.Combined:
            self.img.clear()
            self.gradient_legend.hide()
            try:
                self.removeItem(self.gradient_legend)
                pass
            except Exception, e:
                if not "Could not determine index of item" in str(e):
                    raise
            self.plotItemPlot.clear()
            self.plotItemPlotScatter.setData(data[0].transpose())
            self.plotItemPlotScatter1.setData(data[1].transpose())
            # self.plotItemPlotScatter2.setData(data[2].transpose())  # Commented out for debug, see dataset.py combined()
            self.plotItem.setClipToView(False) # NOTE: otherwise not all lines are drawn (if the corresponding point is out of view)
        if autorange:
            if self.plotType in [PlotType.FFT, PlotType.Heatmap, PlotType.Combined]:
                self.plotItem.getViewBox().autoRange()

        self.labelAxes()

    def labelAxes(self):
        if self.plotType == PlotType.Heatmap:
            self.plotItem.setLabel('bottom', 'Turn', '')
            self.plotItem.setLabel('left', 'Bunch Position', '')
        elif self.plotType == PlotType.FFT:
            self.plotItem.setLabel('left', 'Bunch Position', '')
            self.plotItem.setLabel('bottom', 'Frequency', 'Hz')
        elif self.plotType == PlotType.Trains:
            self.plotItem.setLabel('left', '', '')
            self.plotItem.setLabel('bottom', 'Sample Point', '')
        elif self.plotType == PlotType.Combined:
            self.plotItem.setLabel('left', '', '')
            self.plotItem.setLabel('bottom', '', '')




    def changeType(self, type):
        self.plotType = type
        self.gradient_legend.image_changed()

class PlotWidget(kcgw.KCGWidgets):
    close_signal = QtCore.pyqtSignal()
    change_type_signal = QtCore.pyqtSignal(int, int, int)
    def __init__(self, name=None, unique_id=None, type=None, datatype=None, prefix=None, fName=None, data=None):
        super(PlotWidget, self).__init__()
        if name != None:
            self.theName = name
        if unique_id != None:
            self.theId = unique_id
        if type != None:
            self.theType = type
        if datatype != None:
            self.theDataType = datatype
        if prefix != None:
            self.thePrefix = prefix
        if self.theDataType == FILE:
            self.fName = fName
        if self.theDataType == FILE:
            self.data = data
        else:
            self.data = None

        self.adc = 1
        self.initUI()
        self.setWindowTitle(self.theName + " - " + plotList[type] + " - " + str(self.thePrefix))
        self.plot_widget.changeType(type)
        if self.theDataType == FILE:
            self.plot(type)
        else:
            live_plot_windows.addWindow(self)
            if board.status.last_file is not None:
                self.data = io.read_from_file(board.status.last_file)
                self.plot(type)

    def initUI(self):
        self.plot_widget = SubPlotWidget()
        self.layout = QtGui.QVBoxLayout()
        self.plot_buttons_layout = QtGui.QHBoxLayout()

        self.heatmap_button = self.createButton(text=tr("Button", "Heatmap"), connect=lambda: self.plot(type=0))
        self.fft_button = self.createButton(text=tr("Button", "FFT"), connect=lambda: self.plot(type=1))
        self.trains_button = self.createButton(text=tr("Button", "Trains"), connect=lambda: self.plot(type=2))
        self.combined_button = self.createButton(text=tr("Button", "Combined"), connect=lambda: self.plot(type=3))

        self.adc1 = self.createCheckbox(text="ADC 1", connect=self.change_adc)
        self.adc1.setChecked(True)
        self.adc2 = self.createCheckbox(text="ADC 2", connect=self.change_adc)
        self.adc3 = self.createCheckbox(text="ADC 3", connect=self.change_adc)
        self.adc4 = self.createCheckbox(text="ADC 4", connect=self.change_adc)

        self.plot_buttons_layout.addWidget(self.heatmap_button)
        self.plot_buttons_layout.addWidget(self.fft_button)
        self.plot_buttons_layout.addWidget(self.trains_button)
        self.plot_buttons_layout.addWidget(self.combined_button)

        self.adc_checkbox_layout = QtGui.QHBoxLayout()
        self.adc_checkbox_layout.addWidget(self.adc1)
        self.adc_checkbox_layout.addWidget(self.adc2)
        self.adc_checkbox_layout.addWidget(self.adc3)
        self.adc_checkbox_layout.addWidget(self.adc4)

        self.groupWidget = QtGui.QWidget()
        self.groupWidget.setLayout(self.adc_checkbox_layout)
        self.group = QtGui.QButtonGroup()
        self.group.addButton(self.adc1, 1)
        self.group.addButton(self.adc2, 2)
        self.group.addButton(self.adc3, 3)
        self.group.addButton(self.adc4, 4)
        self.group.setExclusive(True)

        self.from_to_layout = QtGui.QHBoxLayout()
        self.from_spinbox = self.createSpinbox(0, 100000000, interval=100, connect=lambda: self.plot(self.theType))
        self.to_spinbox = self.createSpinbox(0, 100000000, start_value=1000, interval=100, connect=lambda: self.plot(self.theType))
        self.from_to_layout.addStretch()
        self.from_to_layout.addWidget(self.createLabel(tr("Label", "From:")))
        self.from_to_layout.addWidget(self.from_spinbox)
        self.from_to_layout.addWidget(self.createLabel(tr("Label", "To:")))
        self.from_to_layout.addWidget(self.to_spinbox)

        # self.slider_layout = QtGui.QHBoxLayout()
        # self.low_slider = QtGui.QSlider(QtCore.Qt.Horizontal)
        # self.low_slider.setMinimum(0)
        # self.low_slider.setMaximum(1000)
        # self.low_slider.valueChanged.connect(self.low_changed)
        # self.max_slider = QtGui.QSlider(QtCore.Qt.Horizontal)
        # self.max_slider.setMinimum(100)
        # self.max_slider.setMaximum(10000)
        # self.max_slider.valueChanged.connect(self.max_changed)
        # self.slider_layout.addWidget(self.low_slider)
        # self.slider_layout.addWidget(self.max_slider)
        self.layout.addLayout(self.plot_buttons_layout)
        self.layout.addWidget(self.plot_widget)
        self.layout.addWidget(self.groupWidget)
        self.layout.addLayout(self.from_to_layout)
        # self.layout.addLayout(self.slider_layout)
        self.setLayout(self.layout)
    def low_changed(self, x):
        self.plot_widget.img.setLevels([x, self.max_slider.value()])
    def max_changed(self, x):
        self.plot_widget.img.setLevels([self.low_slider.value(), x])

    def change_adc(self):
        self.adc = self.group.checkedId()
        self.change_type_signal.emit(self.theId, self.theType, self.adc)
        self.plot(self.theType)

    def disable_buttons(self, b_bool):
        self.heatmap_button.setDisabled(b_bool)
        self.fft_button.setDisabled(b_bool)
        self.trains_button.setDisabled(b_bool)
        self.combined_button.setDisabled(b_bool)

    def plot(self, type=None):
        try:
            if type == None:
                type = self.theType
            self.do_plot(type)
        except Exception, e:
            # mb = QtGui.QMessageBox(QtGui.QMessageBox.Warning, "Error while plotting", "There was an error plotting data")
            QtGui.QMessageBox.critical(self, tr("Heading", "Error Plotting Data"), tr("Dialog", "There was an error plotting the data, make shure it is valid and try again."))
            raise
            # mb.exec_()
    def plot_live(self, type=None, data=None):
        self.data = data
        try:
            if type == None:
                type = self.theType
            self.do_plot(type, autorange=False)
        except Exception, e:
            print 'error during plot'
    def do_plot(self, type, autorange=True):
        if self.theType != type:
            if type == PlotType.Combined:
                self.groupWidget.hide()
            elif self.theType == PlotType.Combined:
                self.groupWidget.show()
            self.theType = type
            self.setWindowTitle(self.theName + " - " + plotList[type] + " - " + str(self.thePrefix))
            self.change_type_signal.emit(self.theId, type, self.adc)
            self.plot_widget.changeType(type)

        if type == PlotType.FFT:
            self.plot_widget.plot(np.abs(self.data.fft(adc=self.adc, frm=self.from_spinbox.value(),
                                                       to=self.to_spinbox.value(), drop_first_bin=True)).transpose(),
                                  autorange=autorange,
                                  xvalueborders=[self.data.fft_freq_dist(), self.data.fft_max_freq()])
        if type == PlotType.Heatmap:
            self.plot_widget.plot(self.data.heatmap(adc=self.adc, frm=self.from_spinbox.value(), to=self.to_spinbox.value()).transpose(), autorange=autorange)
        if type == PlotType.Trains:
            self.plot_widget.plot(self.data.train(adc=self.adc, frm=self.from_spinbox.value(), to=self.to_spinbox.value()), autorange=autorange)
        if type == PlotType.Combined:
            self.plot_widget.plot(self.data.combined(frm=self.from_spinbox.value(), to=self.to_spinbox.value()), autorange=autorange)

    @QtCore.pyqtSlot(np.ndarray, tuple)
    def fft_done(self, data, xborders):
        if self.theType == 1:
            self.plot_widget.plot(data, xvalueborders=xborders)

    def closeEvent(self, event):
        del self.data
        del self.plot_widget
        super(PlotWidget, self).closeEvent(event)
        if self.theDataType == LIVE:
            live_plot_windows.removeWindow(self)
        self.close_signal.emit()
        del self
