import sys

from PyQt4 import QtGui

import base.kcgwidget as kcgw
from base.backend import board
bif = sys.modules["base.backendinterface"] # reuse backendinterfacemodule since importing at this point will result in an error
from base.groupedelements import Elements
import config
tr = kcgw.tr

__widget_id__ = None

class acquireSettings(kcgw.KCGWidgets):
    def __init__(self, unique_id, parent=None):
        super(acquireSettings, self).__init__()

        self.id = unique_id
        self.par = parent
        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()
        self.outerLayout.addLayout(self.layout)
        self.outerLayout.addStretch(1)
        self.setLayout(self.outerLayout)

        #---------[ Create Labels and corresponding Fields ]---------
        self.numOfOrbitsLabel = self.createLabel(tr("Label", "Number of orbits to observe")) # TODO: Better Text
        self.numOfSkipOrbitsLabel = self.createLabel(tr("Label", "Number os orbits to skip")) # TODO: Better Text
        self.numOfOrbitsSpinbox = self.createSpinbox(1, 10000000, connect=self.on_number_of_orbits_changed) # TODO: does 10000000 as max make sense?
        self.numOfSkipOrbitsSpinbox = self.createSpinbox(1, 100, connect=self.on_number_of_skipped_orbits_changed) # TODO: 100 as max make sense?
        self.countLabel = self.createLabel(tr("Label", "Count"))
        self.countSpinbox = self.createSpinbox(1, 10000000, start_value=10, connect=self.on_count_changed) # TODO: does 10000000 as max make sense?
        self.waitLabel = self.createLabel(tr("Label", "Wait(s)"))
        self.waitSpinbox = self.createSpinbox(1, 60, start_value=15, connect=self.on_wait_changed) # TODO: does 60 as max and default as 15 make sense?
        self.simulatePilotBunch = self.createCheckbox(tr("Button", "Simulate Pilot Bunch"), connect=self.on_simulate_pilot_bunch_changed)
        self.buildSpectrogrammTickbox = self.createCheckbox(tr("Button", "Build Spectograms"), connect=self.on_build_spectrograms_changed)
        # -------[ register observers ]------
        board.config.observe(self.numOfOrbitsSpinbox, lambda x: self.setValueSilent(self.numOfOrbitsSpinbox, x), 'orbits_observe')
        board.config.observe(self.numOfSkipOrbitsSpinbox, lambda x: self.setValueSilent(self.numOfSkipOrbitsSpinbox, x), 'orbits_skip')
        board.config.observe(self.countSpinbox, lambda x: self.setValueSilent(self.countSpinbox, x), 'orbits_count')
        board.config.observe(self.waitSpinbox, lambda x: self.setValueSilent(self.waitSpinbox, x), 'orbits_wait_time')
        board.config.observe(self.simulatePilotBunch, lambda x: self.tickSilent(self.simulatePilotBunch, x), 'pilot_bunch')
        board.config.observe(self.buildSpectrogrammTickbox, lambda x: self.tickSilent(self.buildSpectrogrammTickbox, x), 'build_spectrograms')
        # -------[ register elements ]-------
        Elements.addItem(["acquire", "no_board"],
                         [
                             self.numOfOrbitsSpinbox,
                             self.numOfSkipOrbitsSpinbox,
                             self.countSpinbox,
                             self.waitSpinbox,
                             self.buildSpectrogrammTickbox,
                             self.simulatePilotBunch
                         ])

        # -------[ Add to grid ]-------------
        self.layout.addWidget(self.numOfOrbitsLabel, 0, 0)
        self.layout.addWidget(self.numOfOrbitsSpinbox, 0, 1)
        self.layout.addWidget(self.numOfSkipOrbitsLabel, 1, 0)
        self.layout.addWidget(self.numOfSkipOrbitsSpinbox, 1, 1)
        self.layout.addWidget(self.countLabel, 2, 0)
        self.layout.addWidget(self.countSpinbox, 2, 1)
        self.layout.addWidget(self.waitLabel, 3, 0)
        self.layout.addWidget(self.waitSpinbox, 3, 1)
        self.layout.addWidget(self.simulatePilotBunch, 4, 0)
        self.layout.addWidget(self.buildSpectrogrammTickbox, 4, 1)

        self.setWindowTitle(tr("Heading", "Acquire Settings"))
        self.setValues()


    def setValueSilent(self, element, value):
        element.blockSignals(True)
        element.setValue(value)
        element.blockSignals(False)
    def tickSilent(self, element, state):
        element.blockSignals(True)
        element.setChecked(state)
        element.blockSignals(False)

    def setValues(self):
        self.setValueSilent(self.numOfOrbitsSpinbox, bif.bk_get_config('orbits_observe'))
        self.setValueSilent(self.numOfSkipOrbitsSpinbox, bif.bk_get_config('orbits_skip'))
        self.setValueSilent(self.countSpinbox, bif.bk_get_config('orbits_count'))
        self.setValueSilent(self.waitSpinbox, bif.bk_get_config('orbits_wait_time'))
        self.tickSilent(self.buildSpectrogrammTickbox, bif.bk_get_config('build_spectrograms'))
        self.tickSilent(self.simulatePilotBunch, bif.bk_get_config('pilot_bunch'))

    def on_number_of_orbits_changed(self):
        bif.bk_change_num_of_orbits(self.numOfOrbitsSpinbox.value())
    def on_number_of_skipped_orbits_changed(self):
        bif.bk_change_num_of_skipped_orbits(self.numOfSkipOrbitsSpinbox.value())
    def on_count_changed(self):
        bif.bk_change_count(self.countSpinbox.value())
    def on_wait_changed(self):
        bif.bk_change_wait(self.waitSpinbox.value())
    def on_build_spectrograms_changed(self):
        bif.bk_change_build_spectrograms(self.buildSpectrogrammTickbox.checkState())
    def on_simulate_pilot_bunch_changed(self):
        bif.bk_change_pilot_bunch(self.simulatePilotBunch.checkState())



    def closeEvent(self, event):

        board.config.unobserve(self.numOfOrbitsSpinbox, 'orbits_observe')
        board.config.unobserve(self.numOfSkipOrbitsSpinbox, 'orbits_skip')
        board.config.unobserve(self.countSpinbox, 'orbits_count')
        board.config.unobserve(self.waitSpinbox, 'orbits_wait_time')
        board.config.unobserve(self.simulatePilotBunch, 'pilot_bunch')
        board.config.unobserve(self.buildSpectrogrammTickbox, 'build_spectrograms')
        Elements.removeItem(["acquire", "no_board"],
             [
                 self.numOfOrbitsSpinbox,
                 self.numOfSkipOrbitsSpinbox,
                 self.countSpinbox,
                 self.waitSpinbox,
                 self.buildSpectrogrammTickbox,
                 self.simulatePilotBunch
             ])

        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]
        super(acquireSettings, self).closeEvent(event)


def addAcquireSettingsWidgets():
    global __widget_id__
    if __widget_id__:
        kcgw.area.widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = acquireSettings(nid, kcgw.area)
        kcgw.area.newWidget(w, "Acquire", nid, widget_type=4, minSize=True) #TODO: proper type

def setValueSilent(key, value):
    if not __widget_id__:
        return
    kcgw.area.widgets[__widget_id__].blockSignals(True)
    if key == "orbits_observe":
        kcgw.area.widgets[__widget_id__].numOfOrbitsSpinbox.setValue(value)
    if key == "orbits_skip":
        kcgw.area.widgets[__widget_id__].numOfSkipOrbitsSpinbox.setValue(value)
    kcgw.area.widgets[__widget_id__].blockSignals(False)


kcgw.register_widget(QtGui.QIcon(config.acquireSettingsIcon), tr("Heading", "Acquire Settings"), addAcquireSettingsWidgets, "Ctrl+A")
