from PyQt4 import QtGui, QtCore
import socket
import base.kcgwidget as kcgw
import base.backendinterface as bif
from base.backend import board
from base.groupedelements import Elements

__widget_id__ = None

class Remote(kcgw.KCGWidgets):
    response = QtCore.pyqtSignal(str)
    def __init__(self, unique_id, parent):
        super(Remote, self).__init__()

        self.id = unique_id
        self.par = parent

        self.status = False
        self.send_status = False
        self.host = '0.0.0.0'

        self.layout = QtGui.QGridLayout()
        self.setLayout(self.layout)
        self.start_stop_button_listen = self.createButton("Allow Remote Control", connect=self.start_stop_listen)
        self.start_stop_button_send = self.createButton("Control remote Gui", connect=self.start_stop_send)
        self.port = self.createSpinbox(1024, 65000, interval=1, start_value=1330)
        self.port_label = self.createLabel("Port")
        self.layout.addWidget(self.port_label, 0, 0)
        self.layout.addWidget(self.port, 0, 1)
        self.layout.addWidget(self.start_stop_button_listen, 1, 1)

        self.host_label = self.createLabel("Host")
        self.host_input = self.createInput()
        self.layout.addWidget(self.host_label, 2, 0)
        self.layout.addWidget(self.host_input, 2, 1)
        self.layout.addWidget(self.port_label, 3, 0)
        self.layout.addWidget(self.port, 3, 1)
        self.layout.addWidget(self.start_stop_button_send, 4, 1)

        self.rl = RemoteListener(self.host, self.port.value(), parent=self)
        self.rl.command_received.connect(self.evaluate)
        self.response.connect(self.rl.response)

        self.commands = {
            'calibrate': [["after_start"], bif.bk_calibrate],
            'sync': [["after_start", "synchronize"], bif.bk_sync_board],
            'set_defaults': [['after_start', 'set_defaults'], bif.bk_set_defaults],
            'soft_reset': [['after_start'], bif.bk_soft_reset]
        }
        self.value_types = {
            'fpga_delay_max': int,
            'fpga_delay': int,
            'fpga_delay_factor': int,
            'chip_delay_max': int,
            'chip_1_delay': int,
            'chip_2_delay': int,
            'chip_3_delay': int,
            'chip_4_delay': int,
            'chip_delay_factor': int,
            'th_delay_max': int,
            'th_delay': int,
            'th_delay_factor': int,
            'adc_delay_max': int,
            'adc_1_delay': int,
            'adc_2_delay': int,
            'adc_3_delay': int,
            'adc_4_delay': int,
            'adc_delay_factor': int,
            'th_to_adc_cycles': int,
            'adc_1_delay_individual': int,
            'orbits_observe': int,
            'orbits_skip': int,
            'orbits_count': int,
            'orbits_wait_time': int,
            'build_spectrograms': bool,
            'pilot_bunch': bool,
            'header': bool
        }

    def is_possible(self, groups):
        for group in groups:
            if not Elements.isEnabled(group):
                return False
        return True

    @QtCore.pyqtSlot(str)
    def evaluate(self, command):
        c = str(command).split(" ")
        try:
            if c[0] == 'acquire':
                if self.is_possible(["acquireTrigger"]):
                    bif.bk_acquire()
                    if board.status.acquisition:
                        self.response.emit("Acquisition started - Check status using 'get status'")
                    else:
                        self.response.emit("Acquisition stopped")
                else:
                    self.response.emit("Not Possible")


            if c[0] in self.commands:
                if self.is_possible(self.commands[c[0]][0]):
                    self.response.emit(".Processing")
                    self.commands[c[0]][1]()
                    self.response.emit("Success")
                else:
                    self.response.emit("Not Possible")
            if c[0] == 'set': #  set values in board.config
                board.config.update(c[1], self.value_types[c[1]](c[2]))
                print 'setting', c[1], 'to', c[2]
                self.response.emit("Success")
            if c[0] == 'get':  # get values from board.config
                if c[1] == 'status':
                    if board.status.acquisition:
                        perc = int(board.acquisition_progressbar.value() / float(
                                                                     board.acquisition_progressbar.maximum() -
                                                                     board.acquisition_progressbar.minimum()) * 100)
                        perc = str(perc) + "%"
                    else:
                        perc = "No Acquisition"
                    self.response.emit(str(perc))
                elif c[1] == 'all_acquisition': # get all values important for acquisition
                    resp = "Orbits Observed: " + str(board.config.get("orbits_observe")) + "\n" + \
                           "Orbits Skipped: " + str(board.config.get("orbits_skip")) + "\n" + \
                           "Count: " + str(board.config.get("orbits_count")) + "\n" + \
                           "Wait Time: " + str(board.config.get("orbits_wait_time")) + "\n" + \
                           "Pilot Bunch Simulator: " + str(board.config.get("pilot_bunch")) + "\n" + \
                           "Header: " + str(board.config.get("header"))
                    self.response.emit(resp)
                else:
                    self.response.emit(str(board.config.get(c[1])))

        except Exception, e:
            self.response.emit("Failed \n"+str(e))


    def start_stop_listen(self):
        self.status = not self.status
        self.rl.status = self.status
        if not self.status:
            self.listen_process.terminate()
            self.start_stop_button_listen.setText("Allow Remote Control")
            del self.listen_process
        else:
            # self.listen_process = mp.Process(target=self.rl.listener)
            self.listen_process = QtCore.QThread()
            self.rl.moveToThread(self.listen_process)
            self.listen_process.started.connect(self.rl.listener)
            self.listen_process.start()
            self.start_stop_button_listen.setText("Disallow Remote Control")

    def start_stop_send(self):
        self.send_status = not self.send_status
        if not self.send_status:
            board.safe_call = board.backup_safe_call
            self.start_stop_button_send.setText("Control remote Gui")
        else:
            board.host = self.host_input.text()
            board.port = self.port.value()
            board.safe_call = board.safe_remote_call
            Elements.setEnabled('no_board', True)
            bif.bk_check_for_board()
            bif._bif_status_readout()
            self.start_stop_button_send.setText("Control local Gui")

    def closeEvent(self, event):
        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]

def addRemoteWidget():
    global __widget_id__
    if __widget_id__:
        kcgw.area.widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = Remote(nid, kcgw.area)
        kcgw.area.newWidget(w, "Remote Widget", nid, widget_type=4, minSize=True)

kcgw.register_widget(QtGui.QIcon("icons/project.svg"), "Remote Widget", addRemoteWidget, "Ctrl+r")

class RemoteListener(QtCore.QObject):
    command_received = QtCore.pyqtSignal(str)

    def __init__(self, host, port, parent=None):
        super(RemoteListener, self).__init__()
        self.host = host
        self.port = port
        self.parent = parent
        self.status = True
        self.soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.soc.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.soc.bind((self.host, self.port))
        self.soc.listen(0)
        self.response_text = None

    def evaluate(self, command):
        self.command_received.emit(command)

    def listener(self):

        # conn, addr = soc.accept()
        while self.status:
            try:
                conn, addr = self.soc.accept()
                command = conn.recv(1024)
                if not command: continue
                if command == "stop":
                    break
                self.evaluate(command)
                # while True:
                #     if self.response:
                #         self.soc.send(self.response)
                #     else:
                #         time.sleep(0.01)
                self.wait_signal(self.parent.response, 2000)
                if self.response_text:
                    while self.response_text[0] == '.':
                        conn.send(self.response_text)
                        self.wait_signal(self.parent.response, 2000)
                    conn.send(self.response_text)
                else:
                    conn.send("Failed")

                self.response_text = None

                conn.close()
            except socket.error:  # Go on even in case of exception
                pass
        conn.close()

    def response(self, res):
        self.response_text = res

    def wait_signal(self, signal, timeout=10000):
        """Block loop until signal emitted, or timeout (ms) elapses."""
        loop = QtCore.QEventLoop()
        signal.connect(loop.quit)

        if timeout is not None:
            QtCore.QTimer.singleShot(timeout, loop.quit)
        loop.exec_()

