# -*- coding: utf-8 -*-
from PyQt4 import QtGui

import base.kcgwidget as kcgw
import base.backendinterface as bif
from base.groupedelements import Elements
import config

tr = kcgw.tr

__widget_id__ = None

class singleReadWidget(kcgw.KCGWidgets):
    def __init__(self, unique_id, parent=None):
        super(singleReadWidget, self).__init__()

        self.id = unique_id
        self.par = parent
        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()
        self.outerLayout.addLayout(self.layout)
        self.outerLayout.addStretch(1)
        self.setLayout(self.outerLayout)
        self.continuous_read = False

        #---------[ Create Labels and corresponding Fields ]---------
        self.single_read_button = self.createButton(tr("Button", "Single Read"), connect=bif.bk_single_read)
        self.continuous_read_button = self.createButton(tr("Button", "Start Continuous Read"), connect=self.on_continuous_read)
        self.interval_spinbox = self.createSpinbox(0, 100000, start_value=1000)
        Elements.addItem(["acquire", "no_board"], [self.single_read_button, self.continuous_read_button])
        Elements.addItem("continuous_read", self.continuous_read_button)

        #--------[ Fill Grid ]----------------
        self.layout.addWidget(self.single_read_button, 0, 0)
        self.layout.addWidget(self.continuous_read_button, 0, 1)
        self.layout.addWidget(self.createLabel(tr("Label", "Interval:") + " (ms)"), 1, 0)
        self.layout.addWidget(self.interval_spinbox, 1, 1)
        self.setWindowTitle(tr("Heading", "Single and Continuous Read"))
    def on_continuous_read(self):
        if not self.continuous_read:
            self.continuous_read_button.setText(tr("Button", "Stop Continuous Read"))
            self.continuous_read = True
        else:
            self.continuous_read_button.setText(tr("Button", "Start Continuous Read"))
            self.continuous_read = False
        bif.bk_continuous_read(interval=self.interval_spinbox.value())

    def closeEvent(self, event):
        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]
        Elements.removeItem(['no_board', 'acquire'],
                            [
                                self.single_read_button,
                                self.continuous_read_button
                            ])

        super(singleReadWidget, self).closeEvent(event)


def addSingleReadWidget():
    global __widget_id__
    if __widget_id__:
        kcgw.area.widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = singleReadWidget(nid, kcgw.area)
        kcgw.area.newWidget(w, "Single and Continuous Read", nid, widget_type=4, minSize=True) #TODO: proper type

kcgw.register_widget(QtGui.QIcon(config.singleReadIcon), tr("Heading", "Single Read"), addSingleReadWidget, "Ctrl+i")
